# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008, 2011 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Bernhard Herzog <bernhard.herzog@intevation.de>
#

from formed.renderer.html import HtmlRenderer, UrlFactory as BaseUrlFactory
from formed.renderer.html import RendererContext, Container, Item
from formed.renderer.helpers import tag, escape

from mpulsweb.lib.helpers import url


class UrlFactory(BaseUrlFactory):

    def url(self, controller, action, id=None):
        url_args= {"controller": controller, "action":action}
        if not id is None:
            url_args["id"] = id
        return url(**url_args)

    def icon(self, name):
        """Return the url for the icon given by name."""
        return url("/images/icons/" + name)


class ViewRenderer(HtmlRenderer):

    def error_info(self, context, nc):
        messages = self.data.get_errors(nc)
        if messages:
            return ("error", messages)

        messages = self.get_warnings(nc)
        if messages:
            return ("warning", messages)

        return ("", [])

class ViewPrintRenderer(ViewRenderer):

    def render_group(self, context, nc):
        description = nc.getDescription()

        out = []
        if nc.isInvisible():
            if description:
                out.append("<!-- %s -->\n" % escape(description))
        else:
            out.append("<table border=\"1\">\n")
            if description:
                out.append("<th>%s %s</th>\n"
                           % ("".join(self.render_label(context, nc,
                                                        self.data.get_id(nc))),
                              "".join(self.render_annotation_links(nc))))
            out.append("<tr><td>\n")
            out.append(self.render_children(RendererContext(nc, Container), nc))
            out.append("</td></tr></table>\n")
        return Item("".join(out))

    def render_inline_repeat_group(self, context, nc):
        description = nc.getDescription()
        out = []
        out.append('<table border=\"1\">')
        out.append('<th>%s %s</th>'
                   % (escape(description),
                      "".join(self.render_annotation_links(nc))))
        out.append('<tr><td>')
        out.append('<table class="repeatgroup">')
        out.extend(self.render_repeat_list(context, nc))
        out.append('</table>')
        out.append('</td></tr></table>')
        return Item("".join(out))
