SET ROLE postgres;

ALTER FUNCTION mpuls_revoke_connect( varchar, varchar ) OWNER TO :adm_ka_owner;
GRANT EXECUTE ON FUNCTION  mpuls_revoke_connect( varchar, varchar )
    TO GROUP mpuls_role_adm;

REVOKE ALL ON SCHEMA public FROM PUBLIC;
GRANT USAGE ON SCHEMA public TO :cm_group;
GRANT USAGE ON SCHEMA public TO :admin_group;
GRANT USAGE ON SCHEMA public TO mpuls_role_adm;
GRANT ALL ON SCHEMA public TO :adm_ka_owner;

SET ROLE :adm_ka_owner;
BEGIN;

-- Allow users to create temporary tables (needed for evaluations)
GRANT TEMP ON DATABASE :db_name TO PUBLIC;

CREATE TABLE dbinfo (
    id INTEGER PRIMARY KEY NOT NULL,
    key   VARCHAR(128) NOT NULL,
    value VARCHAR(256)
);

INSERT INTO dbinfo (id, key, value) VALUES (1, 'version', CURRENT_DATE);
INSERT INTO dbinfo (id, key, value) VALUES (2, 'datadump', NULL);

GRANT SELECT ON dbinfo TO PUBLIC;

-- Trigger/Logging

-- Protokolliert werden:
-- - Zeitstempel (über DB Konfiguration, vgl. log_line_prefix)
-- - Datenbank (über DB Konfiguration, vgl. log_line_prefix)
-- - ID des Datensatz
-- - Aktion
-- - Tabelle
-- - User
--
-- Nicht protokolliert werden die genauen Eingaben. Diese lassen sich bei
-- Bedarf aus dem Write-Ahead-Log rekonstruieren, nachdem in den
-- Protokolldateien der Zeitpunkt identifiziert wurde.

CREATE OR REPLACE FUNCTION mpuls_log() RETURNS TRIGGER AS $$
BEGIN
    IF TG_OP = 'DELETE' THEN
        RAISE INFO '% on % (id: %) by %', TG_OP, TG_TABLE_NAME, OLD.id, session_user;
        RETURN OLD;
    ELSE
        RAISE INFO '% on % (id: %) by %', TG_OP, TG_TABLE_NAME, NEW.id, session_user;
        RETURN NEW;
    END IF;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION rg_lastaccess_log() RETURNS trigger AS $rg_lastaccess_log$
BEGIN
IF TG_OP = 'DELETE' THEN
    EXECUTE set_lastaccess_time(OLD.master_id);
    RETURN OLD;
ELSE
    EXECUTE set_lastaccess_time(NEW.master_id);
    RETURN NEW;
END IF;
END;
$rg_lastaccess_log$ LANGUAGE plpgsql;

COMMIT;
