SET ROLE :adm_ka_owner;
BEGIN;

CREATE TABLE ka_termine_tbl
(
        id serial NOT NULL,
        master_id int4 REFERENCES master_tbl(id) ON DELETE CASCADE,
        sdatum timestamp,
        edatum timestamp,
        name varchar(128),
        beschreibung text,
        art int4 DEFAULT 0,
        CONSTRAINT ka_termine_tbl_pkey PRIMARY KEY (id)
);

-- Funktionen zum Erstellen und Löschen von Terminen
CREATE OR REPLACE FUNCTION create_fall_termin_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    INSERT INTO ka_termine_tbl(master_id) VALUES (mid);
    RETURN currval('ka_termine_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_termin_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_termin_ds( integer )
                TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_fall_termin_ds(tid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM ka_termine_tbl WHERE id = tid;
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;
    DELETE FROM ka_termine_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_fall_termin_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_fall_termin_ds(integer)
                TO GROUP :cm_group;

-- Funktion, die alle Termine zu einem gegebenen Fall loescht
CREATE OR REPLACE FUNCTION delete_all_fall_termine(mid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
BEGIN
    DELETE FROM ka_termine_tbl WHERE master_id = mid;
END;
$$;

REVOKE ALL     ON FUNCTION delete_all_fall_termine(integer) FROM PUBLIC;
GRANT  EXECUTE ON FUNCTION delete_all_fall_termine(integer) TO GROUP :admin_group;

-- Funktionen zum Anlegen und Löschen von globalen Terminen
CREATE OR REPLACE FUNCTION create_global_termin_ds() RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO ka_termine_tbl (master_id) VALUES (NULL);
    RETURN currval('ka_termine_tbl_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_global_termin_ds() FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_global_termin_ds() TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_global_termin_ds(tid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM ka_termine_tbl WHERE id = tid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_global_termin_ds(tid integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_global_termin_ds( integer )
            TO GROUP :admin_group;

-- Lieftert das letzte relevante "Datum" zu einem durch den mit
-- "mid" identifiezierten Fall.
-- Die Menge der durchsuchten Termine sind die
-- fallspezifischen Termine sowie das letze Beabreitungsdatum.
CREATE OR REPLACE FUNCTION getlastcaseappointment(mid int4)
        RETURNS "timestamp" AS
$BODY$
DECLARE
        maxdate timestamp;
BEGIN
        SELECT INTO maxdate max(date::timestamp) FROM (
        SELECT zugriff as date from
        ka_status_tbl_view
        WHERE master_id = mid
        UNION
        SELECT max(sdatum) as date from
        ka_fall_termine_tbl_view
        WHERE master_id = mid
        UNION
        SELECT max(edatum) as date from
        ka_fall_termine_tbl_view
        WHERE master_id = mid
        ) as test;
        RETURN maxdate;
END;
$BODY$ LANGUAGE 'plpgsql' VOLATILE;
REVOKE ALL ON FUNCTION getlastcaseappointment(int) FROM public;
GRANT EXECUTE ON FUNCTION getlastcaseappointment(int)  TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION getlastcaseappointment(int)  TO GROUP :admin_group;

-- Trigger Logging
CREATE TRIGGER t_ka_termine_tbl_insert AFTER
    INSERT ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t_ka_termine_tbl_update AFTER
    UPDATE ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t_ka_termine_tbl_delete AFTER
    DELETE ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE mpuls_log();

CREATE TRIGGER t1_ka_termine_tbl_insert AFTER
    INSERT ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_termine_tbl_update AFTER
    UPDATE ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

CREATE TRIGGER t1_ka_termine_tbl_delete AFTER
    DELETE ON ka_termine_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_lastaccess_log();

COMMIT;
-- Create Views.
\i sql/install/views/07_appointments.sql
