SET ROLE :adm_ka_owner;
BEGIN;

-- Funktion get_default_value() gibt zu einer
-- gegebenen Relation und Spalte den Default-Wert zurueck gibt.

CREATE OR REPLACE FUNCTION get_default_value(relation_name text, attribute_name text)
RETURNS text
LANGUAGE plpgsql STABLE AS
$$
DECLARE
    has_def bool;
    def     text;
BEGIN
    SELECT INTO has_def a.atthasdef FROM pg_class c
    JOIN pg_attribute a ON c.oid = a.attrelid
    WHERE c.relname = relation_name AND a.attname = attribute_name;

    IF has_def IS NULL OR NOT has_def THEN
        RETURN NULL;
    END IF;

    SELECT INTO def pg_get_expr(i.adbin, i.oid) FROM
    (SELECT c.oid, adnum, r.adbin
        FROM pg_class c
        JOIN pg_attrdef r ON r.adrelid = c.oid
        WHERE c.relname = relation_name) i
        INNER JOIN pg_attribute p ON attrelid = i.oid AND i.adnum = p.attnum
        WHERE p.attname = attribute_name;
    RETURN def;
END;
$$;

REVOKE ALL ON FUNCTION get_default_value(text, text) FROM public;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION get_default_value(text, text) TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION set_last_logintime(userid integer)
    RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    UPDATE ka_benutzer_tbl_view SET last_login = now() WHERE id = userid;
END;
$$ ;

REVOKE ALL ON FUNCTION set_last_logintime(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION set_last_logintime(integer) TO GROUP :admin_group;

COMMIT;
