# -*- coding: latin-1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from xml.sax.handler import ContentHandler


class Creator:

    def createFactory(self, tag, attrs, context):
        return None


class Factory:

    def eat(self, element, context):
        pass

    def reduce(self, context):
        return None


class ArrayFactory(Factory):

    def __init__(self):
        self.array = []

    def eat(self, element, context):
        self.array.append(element)

    def reduce(self, context):
        return self.array


class SAXBuilder(ContentHandler):

    def __init__(self, context=None):
        self.context = context
        self.creators = {}
        self.stack = []
        self.document = None

    def getDocument(self):
        return self.document

    def addCreator(self, name, creator):
        self.creators[name] = creator

    def setCreators(self, creators):
        self.creators = creators

    def startElement(self, name, attrs):
        creator = self.creators[name]
        factory = creator.createFactory(name, attrs, self.context)
        self.stack.append(factory)

    def endElement(self, name):
        factory = self.stack.pop()
        self.stack[-1].eat(factory.reduce(self.context), self.context)

    def startDocument(self):
        self.stack = [ArrayFactory()]

    def endDocument(self):
        self.document = self.stack.pop().reduce(self.context)[0]

    def characters(self, content):
        self.stack[-1].eat(content, self.context)

# vim:set ts=4 sw=4 si et sta sts=4:
