#!/bin/sh

usage()
{
cat << EOF
usage: $0 options source

This script will install the distribution under /home/mpuls/releases/<name>-<rev>.
It will further setup a virtual environment.
Database setup is not done!

OPTIONS:
   -h       Show this message
   -n       Name of the release (Defaults to name of archived source-revision)
   -r       Revision number used to build the release (Default: tip of working directory)
   -v       Setup a virtual environment for python and install the packages in it (Default: No).
   -a       Make the installed package the active package by setting the "current" sybolic link (Default: No).
EOF
}

# DEFAULTS
VERBOSE=0
INSTALL_BASE_PATH=/home/mpuls

NAME=
BUILDDIR=/tmp
REVISION=
COMPRESS=0
ACTIVATE=0
VIRTUALPYTHON=0

# Parse Options
while getopts "hvan:r:" OPTION
do
        case $OPTION in
            h)
                usage
                exit 1
                ;;
            a)
                ACTIVATE=1
                ;;
            n)
                NAME=$OPTARG
                ;;
            r)
                REVISION=$OPTARG
                ;;
            v)
                VIRTUALPYTHON=1
                ;;
            ?)
                # Will be set if the optstring begins with a ":". In this case
                # all options not defined in the optstring will OPTION set to
                # "?". Otherwise the shell reports an error on the unknown
                # option.
                usage
                exit
                ;;
        esac
done


# Check if installation base dir exists
if [ ! -d $INSTALL_BASE_PATH ]
then
    echo "Error! Installation directory ${INSTALL_PATH} does not exist!";
    exit 1;
fi


if [ -z ${REVISION} ]
then
    echo "Error! Please provide the revision of the distribution!";
    usage;
    exit 1;
fi

if [ -z $NAME ]
then
    echo "Error! Please provide name of the distribution!";
    usage;
    exit 1;
fi

DISTRIBUTION=`readlink -f ../`
DISTRIBUTION_BASENAME=`basename $DISTRIBUTION`

# Try to figure out names based on package name
#DISTRIBUTION_REV=`basename $DISTRIBUTION | cut -d "-" -f 3` 
#DISTRIBUTION_NAME=`basename $DISTRIBUTION | cut -d "-" -f 1,2`
DISTRIBUTION_REV=$REVISION} 
DISTRIBUTION_NAME=${NAME}

INSTALL_APP_PATH=$INSTALL_BASE_PATH/$DISTRIBUTION_NAME

# Check if installation distribution dir exists
if [ ! -d $INSTALL_APP_PATH ]
then
    echo "Can not find installation directory ${INSTALL_APP_PATH}. Creating it ...";
    mkdir ${INSTALL_APP_PATH}
    cd ${INSTALL_APP_PATH}
    mkdir releases
fi

# Copy all files.
echo "Copying file from ${DISTRIBUTION} to ${INSTALL_APP_PATH}/releases ...";
cd ${INSTALL_APP_PATH}/releases
rm -rf ${DISTRIBUTION_BASENAME}
cp -r ${DISTRIBUTION} .

if [ $ACTIVATE = 1 ]
then
    rm -f current
    ln -s ${DISTRIBUTION_BASENAME} current	
else
    echo "Not activating the installation. Do not to set the current symbolic link to activate it.";
fi

# Set permissions
echo "Setting permissions ...";
chown mpuls: -R ${DISTRIBUTION_BASENAME}
chgrp postgres -R ${DISTRIBUTION_BASENAME}/db-admin
chmod g+w -R ${DISTRIBUTION_BASENAME}/db-admin

# Setup environment
if [ $VIRTUALPYTHON = 1 ]
then
    echo "Setting up virtual environment ...";
    cd ${DISTRIBUTION_BASENAME}/maintenance
    sh setup-environment.sh -n ${DISTRIBUTION_NAME} -g -d .. .. 1>/dev/null
else
    echo "Not setting up a virtual environment";
fi

echo Install finished in ${INSTALL_APP_PATH}/releases/${DISTRIBUTION_BASENAME}!
